// SPDX-License-Identifier: GPL-2.0-or-later

static const ActionDef layout_image_actions[] =
{
		{
		"win.layout-image-alter",
		nullptr,
		N_("Orientation"),
		layout_image_pop_menu_alter_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-animate",
		nullptr,
		N_("GIF animation"),
		layout_image_pop_menu_animate_cb,
		nullptr,
		ActionStateType::BOOLEAN,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-collections",
		nullptr,
		N_("Add to Collection"),
		layout_image_pop_menu_collections_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-copy",
		GQ_ICON_COPY,
		N_("Copy…"),
		layout_image_pop_menu_copy_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-copy-image",
		nullptr,
		N_("Copy image to clipboard"),
		layout_image_pop_menu_copy_image_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-copy-path",
		nullptr,
		N_("Copy to clipboard"),
		layout_image_pop_menu_copy_path_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-copy-path-unquoted",
		nullptr,
		N_("Copy to clipboard (unquoted)"),
		layout_image_pop_menu_copy_path_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-cut-path",
		nullptr,
		N_("Cut to clipboard"),
		layout_image_pop_menu_cut_path_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-delete",
		GQ_ICON_DELETE,
		N_("Move to Trash"),
		layout_image_pop_menu_delete_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-delete-permanent",
		GQ_ICON_DELETE_SHRED,
		N_("Delete"),
		layout_image_pop_menu_delete_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-fullscreen",
		GQ_ICON_FULLSCREEN,
		N_("Fullscreen"),
		layout_image_pop_menu_fullscreen_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-go-to-directory",
		nullptr,
		N_("Go to directory view"),
		layout_image_pop_menu_set_layout_path_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-hide-selectable-toolbars",
		nullptr,
		N_("Hide Selectable Bars"),
		layout_image_pop_menu_hide_selectable_toolbars_cb,
		nullptr,
		ActionStateType::BOOLEAN,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-hide-tools",
		nullptr,
		N_("Hide file list"),
		layout_image_pop_menu_hide_tools_cb,
		nullptr,
		ActionStateType::BOOLEAN,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-move",
		nullptr,
		N_("Move…"),
		layout_image_pop_menu_move_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-open-archive",
		GQ_ICON_OPEN,
		N_("Open archive"),
		layout_image_pop_menu_open_archive_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-plugin-run",
		nullptr,
		N_("Run Plugin"),
		layout_image_pop_menu_plugin_run_cb,
		G_VARIANT_TYPE_STRING,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-rename",
		nullptr,
		N_("Rename…"),
		layout_image_pop_menu_rename_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-slideshow",
		nullptr,
		N_("Toggle slideshow"),
		layout_image_pop_menu_slideshow_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-slideshow-pause",
		nullptr,
		N_("Pause slideshow"),
		layout_image_pop_menu_slideshow_pause_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-view-in-new-window",
		GQ_ICON_NEW,
		N_("View in new window"),
		layout_image_pop_menu_view_new_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-zoom-1-1",
		GQ_ICON_ZOOM_100,
		N_("Zoom 1:1"),
		layout_image_pop_menu_zoom_set_cb<1>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-zoom-fit",
		GQ_ICON_ZOOM_FIT,
		N_("Zoom to fit"),
		layout_image_pop_menu_zoom_set_cb<0>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-zoom-in",
		GQ_ICON_ZOOM_IN,
		N_("Zoom in"),
		layout_image_pop_menu_zoom_in_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.layout-image-zoom-out",
		GQ_ICON_ZOOM_OUT,
		N_("Zoom out"),
		layout_image_pop_menu_zoom_out_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		}
};
