// SPDX-License-Identifier: GPL-2.0-or-later

static const ActionDef view_file_actions[] =
{
		{
		"win.view-file-collections",
		nullptr,
		N_("Collection"),
		vf_pop_menu_collections_action_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-copy",
		GQ_ICON_COPY,
		N_("Copy…"),
		vf_pop_menu_copy_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-copy-path",
		nullptr,
		N_("Copy path"),
		vf_pop_menu_copy_path_action_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-copy-path-unquoted",
		nullptr,
		N_("Copy path unquoted"),
		vf_pop_menu_copy_path_action_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-cut-path",
		nullptr,
		N_("Cut path"),
		vf_pop_menu_cut_path_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-delete",
		GQ_ICON_DELETE,
		N_("Move selection to trash"),
		vf_pop_menu_delete_action_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-delete-permanent",
		GQ_ICON_DELETE_SHRED,
		N_("Delete selection"),
		vf_pop_menu_delete_action_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-disable-grouping",
		nullptr,
		N_("Disable file grouping"),
		vf_pop_menu_disable_grouping_action_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-duplicates",
		GQ_ICON_FIND,
		N_("Find duplicates…"),
		vf_pop_menu_duplicates_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-enable-grouping",
		nullptr,
		N_("Enable file grouping"),
		vf_pop_menu_disable_grouping_action_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-mark-to-selection",
		nullptr,
		N_("Mark to selection"),
		vf_pop_menu_mark_to_selection_action_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-move",
		nullptr,
		N_("Move…"),
		vf_pop_menu_move_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-open-archive",
		GQ_ICON_OPEN,
		N_("Open archive"),
		vf_pop_menu_open_archive_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-plugin-run",
		nullptr,
		N_("Run Plugin"),
		vf_pop_menu_edit_action_cb,
		G_VARIANT_TYPE_STRING,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-refresh",
		GQ_ICON_REFRESH,
		N_("Refresh"),
		vf_pop_menu_refresh_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-rename",
		nullptr,
		N_("Rename…"),
		vf_pop_menu_rename_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-selection-to-mark",
		nullptr,
		N_("Selection to mark"),
		vf_pop_menu_selection_to_mark_action_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-show-filename-text",
		nullptr,
		N_("Show filename text"),
		vf_pop_menu_show_filename_text_action_cb,
		nullptr,
		ActionStateType::BOOLEAN,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-show-star-rating",
		nullptr,
		N_("Show star rating"),
		vf_pop_menu_show_star_rating_action_cb,
		nullptr,
		ActionStateType::BOOLEAN,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-show-thumbnails",
		nullptr,
		N_("Show thumbnails"),
		vf_pop_menu_show_thumbnails_action_cb,
		nullptr,
		ActionStateType::BOOLEAN,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-sort",
		nullptr,
		N_("Sort"),
		vf_pop_menu_sort_action_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::INT32,
		FALSE,
		nullptr,
		SORT_NAME,
		nullptr,
		},

		{
		"win.view-file-sort-ascending",
		nullptr,
		N_("Ascending"),
		vf_pop_menu_sort_ascending_action_cb,
		nullptr,
		ActionStateType::BOOLEAN,
		TRUE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-sort-case",
		nullptr,
		N_("Case"),
		vf_pop_menu_sort_case_action_cb,
		nullptr,
		ActionStateType::BOOLEAN,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-view-new",
		GQ_ICON_NEW,
		N_("View in new window"),
		vf_pop_menu_view_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.view-file-view-type",
		nullptr,
		N_("View type"),
		vf_pop_menu_view_type_action_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::INT32,
		FALSE,
		nullptr,
		FILEVIEW_LIST,
		nullptr,
		},

		{
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr
		}
};
