/*
 * Copyright (C) 2004 John Ellis
 * Copyright (C) 2008 - 2017 The Geeqie Team
 *
 * Author: Colin Clark
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef TOOLBAR_H
#define TOOLBAR_H

#include <gtk/gtk.h>

struct LayoutWindow;

enum ToolbarType : gint {
	TOOLBAR_MAIN,
	TOOLBAR_STATUS,
	TOOLBAR_COUNT
};

GtkWidget *toolbar_select_new(LayoutWindow *lw, ToolbarType bar);
void toolbar_apply(ToolbarType bar);

const gchar *toolbar_type_config_name(ToolbarType type);
void toolbarlist_add_cb(GSimpleAction *, GVariant *, gpointer data);
void toolbarlist_add_button_cb(GSimpleAction *, GVariant *parameter, gpointer data);

#endif

/* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
