// SPDX-License-Identifier: GPL-2.0-or-later

static const ActionDef search_actions[] =
{
		{
		"win.search-win-collections",
		nullptr,
		N_("Collection"),
		search_pop_menu_collections_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-collection-from-selection",
		nullptr,
		N_("Collection from selection"),
		search_result_collection_from_selection_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-copy",
		nullptr,
		N_("Copy…"),
		sr_menu_copy_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-copy-path",
		nullptr,
		N_("Copy path"),
		sr_menu_copy_path_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-copy-path-unquoted",
		nullptr,
		N_("Copy path unquoted"),
		sr_menu_copy_path_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-delete",
		nullptr,
		N_("Delete"),
		sr_menu_delete_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-delete-permanent",
		nullptr,
		N_("Delete"),
		sr_menu_delete_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-help",
		nullptr,
		N_("Help"),
		search_window_help_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-move",
		nullptr,
		N_("Move…"),
		sr_menu_move_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-play",
		nullptr,
		N_("Play"),
		sr_menu_play_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-plugin-run",
		nullptr,
		N_("Run Plugin"),
		sr_menu_edit_cb,
		G_VARIANT_TYPE_STRING,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-print",
		nullptr,
		N_("Print…"),
		sr_menu_print_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-remove-selection",
		nullptr,
		N_("Remove selection"),
		search_remove_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-rename",
		nullptr,
		N_("Rename…"),
		sr_menu_rename_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-result-clear",
		nullptr,
		N_("Result clear"),
		search_win_result_clear_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-result-menu",
		nullptr,
		N_("Search result menu"),
		search_result_menu_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-search-start",
		nullptr,
		N_("Search start"),
		search_start_action_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-select-all",
		nullptr,
		N_("Select all"),
		sr_menu_select_all_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-select-none",
		nullptr,
		N_("Select none"),
		sr_menu_select_none_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-thumbnails",
		nullptr,
		N_("Thumbnails"),
		search_thumbnails_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-view",
		nullptr,
		N_("View"),
		sr_menu_view_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-view-in-new-window",
		nullptr,
		N_("View in new window"),
		sr_menu_viewnew_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.search-win-window-close",
		nullptr,
		N_("Close window"),
		search_window_close_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},


		{
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr
		}

};
