const ActionDef app_actions[] =
{
		{
		"app.about",
		GQ_ICON_ABOUT,
		N_("About"),
		layout_menu_about_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.help-change-log",
		nullptr,
		N_("ChangeLog notes"),
		layout_menu_changelog_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.help-contents",
		GQ_ICON_HELP,
		N_("Help manual (html)"),
		layout_menu_help_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.help-keyboard",
		nullptr,
		N_("Keyboard map"),
		layout_menu_kbd_map_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.help-notes",
		"text-x-readme",
		N_("Readme"),
		layout_menu_notes_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.help-pdf",
		GQ_ICON_HELP,
		N_("Help manual (pdf)"),
		layout_menu_help_pdf_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.help-search",
		nullptr,
		N_("On-line help search"),
		layout_menu_help_search_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.help-shortcuts",
		nullptr,
		N_("Keyboard shortcuts"),
		layout_menu_help_keys_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.new-collection",
		PIXBUF_INLINE_COLLECTION,
		N_("New collection"),
		layout_menu_new_collection_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.new-window-default",
		nullptr,
		N_("New window (default)"),
		layout_menu_window_default_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.open-collection",
		GQ_ICON_OPEN,
		N_("Open collection…"),
		layout_menu_open_collection_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.open-file",
		GQ_ICON_OPEN,
		N_("Open file…"),
		layout_menu_open_file_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.open-recent-file",
		nullptr,
		N_("Open recent file…"),
		layout_menu_open_recent_file_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.open-with",
		GQ_ICON_OPEN_WITH,
		N_("Open With…"),
		layout_menu_open_with_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.preferences",
		GQ_ICON_PREFERENCES,
		N_("Preferences…"),
		layout_menu_config_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"app.quit",
		GQ_ICON_QUIT,
		N_("Quit"),
		layout_menu_exit_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},
};
