// SPDX-License-Identifier: GPL-2.0-or-later

#ifndef CONVERT_CONFIGURATION_H
#define CONVERT_CONFIGURATION_H

void convert_accel_map();
void convert_configuration_file();

#endif
/* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
