// SPDX-License-Identifier: GPL-2.0-or-later

const ActionDef image_actions[] =
{
		{
		"win.image-win-close-window",
		nullptr,
		N_("Close window"),
		view_close_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-collections",
		nullptr,
		N_("Collections"),
		image_pop_menu_collections_cb,
		G_VARIANT_TYPE_INT32,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-copy",
		GQ_ICON_COPY,
		N_("Copy…"),
		view_copy_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-copy-path",
		nullptr,
		N_("Copy path"),
		view_copy_path_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-copy-path-unquoted",
		nullptr,
		N_("Copy path unquoted"),
		view_copy_path_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-delete",
		nullptr,
		N_("Move to rubbish bin"),
		view_delete_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-delete-permanent",
		nullptr,
		N_("Delete"),
		view_delete_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-escape",
		GQ_ICON_GENERIC,
		N_("Toggle fullscreen/Close window"),
		view_escape_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-flip",
		GQ_ICON_FLIP_VERTICAL,
		N_("Flip"),
		view_alter_cb<ALTER_FLIP>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-fullscreen",
		nullptr,
		N_("Fullscreen"),
		view_fullscreen_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-go-to-directory-view",
		nullptr,
		N_("Go to directory view"),
		view_set_layout_path_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-menu",
		nullptr,
		N_("Image menu"),
		view_help_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-mirror",
		GQ_ICON_FLIP_HORIZONTAL,
		N_("Mirror"),
		view_alter_cb<ALTER_MIRROR>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-move",
		nullptr,
		N_("Move…"),
		view_move_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-orientation-original",
		PIXBUF_INLINE_ICON_ORIGINAL,
		N_("Original state"),
		view_alter_cb<ALTER_NONE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-pause-slideshow",
		nullptr,
		N_("Pause slideshow"),
		view_slideshow_pause_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-plugin-run",
		nullptr,
		N_("Run Plugin"),
		view_edit_cb,
		G_VARIANT_TYPE_STRING,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-print",
		nullptr,
		N_("Print"),
		view_print_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-reload",
		nullptr,
		N_("Image reload"),
		view_reload_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-rename",
		nullptr,
		N_("Rename…"),
		view_rename_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-rotate-180",
		PIXBUF_INLINE_ICON_180,
		N_("Rotate 180°"),
		view_alter_cb<ALTER_ROTATE_180>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-rotate-90-ccw",
		GQ_ICON_ROTATE_LEFT,
		N_("Rotate counterclockwise 90°"),
		view_alter_cb<ALTER_ROTATE_90_CC>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-rotate-90-cw",
		GQ_ICON_ROTATE_RIGHT,
		N_("Rotate clockwise 90°"),
		view_alter_cb<ALTER_ROTATE_90>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-set-desaturate",
		nullptr,
		N_("Desaturate"),
		image_set_desaturate_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-slideshow-pause-toggle",
		GQ_ICON_GENERIC,
		N_("Toggle slideshow"),
		view_slideshow_pause_toggle_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-slideshow-start-stop",
		GQ_ICON_GENERIC,
		N_("Toggle slideshow"),
		view_slideshow_start_stop_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-step-first",
		nullptr,
		N_("First"),
		view_step_to_end<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-step-last",
		nullptr,
		N_("Last"),
		view_step_to_end<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-step-next",
		nullptr,
		N_("Next"),
		view_step_next_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-step-previous",
		nullptr,
		N_("Previous"),
		view_step_prev_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-view-fullscreen-toggle",
		GQ_ICON_GENERIC,
		N_("Toggle fullscreen"),
		view_fullscreen_toggle_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-view-help",
		GQ_ICON_GENERIC,
		N_("Help"),
		view_help_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-view-in-new-window",
		nullptr,
		N_("View in new window"),
		view_new_window_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-view-overlay-toggle",
		GQ_ICON_GENERIC,
		N_("Toggle overlay"),
		view_overlay_toggle,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-1-1",
		GQ_ICON_GENERIC,
		N_("Zoom 1:1"),
		view_zoom_set_cb<1>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-1-1",
		GQ_ICON_GENERIC,
		N_("Zoom 1:1"),
		view_zoom_set_cb<1>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-1-2",
		GQ_ICON_GENERIC,
		N_("Zoom 1:2"),
		view_zoom_set_cb<2>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-1-3",
		GQ_ICON_GENERIC,
		N_("Zoom 1:3"),
		view_zoom_set_cb<-3>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-1-4",
		GQ_ICON_GENERIC,
		N_("Zoom 1:2"),
		view_zoom_set_cb<-4>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-2-1",
		GQ_ICON_GENERIC,
		N_("Zoom 2:1"),
		view_zoom_set_cb<2>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-3-1",
		GQ_ICON_GENERIC,
		N_("Zoom 3:1"),
		view_zoom_set_cb<3>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-4-1",
		GQ_ICON_GENERIC,
		N_("Zoom 4:1"),
		view_zoom_set_cb<4>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-fill-horizontal",
		GQ_ICON_GENERIC,
		N_("Zoom fill horizontal"),
		view_zoom_set_fill_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-fill-vertical",
		GQ_ICON_GENERIC,
		N_("Zoom fill vertical"),
		view_zoom_set_fill_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-in",
		nullptr,
		N_("Zoom in"),
		view_zoom_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-in",
		nullptr,
		N_("Zoom in"),
		view_zoom_cb<TRUE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-out",
		nullptr,
		N_("Zoom out"),
		view_zoom_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-out",
		nullptr,
		N_("Zoom out"),
		view_zoom_cb<FALSE>,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},

		{
		"win.image-win-zoom-to-fit",
		nullptr,
		N_("Zoom to fit"),
		view_zoom_fit_cb,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr,
		},
		{
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		nullptr,
		ActionStateType::NONE,
		FALSE,
		nullptr,
		0,
		nullptr
		},
};
