/*
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef SPELL_H
#define SPELL_H

#include <gtk/gtk.h>

void spell_text_view_enable(GtkTextView *text_view);

#endif /* SPELL_H */

/* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
