/*
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef KEYBOARD_SHORTCUTS_H
#define KEYBOARD_SHORTCUTS_H

void shortcuts_window_new_from_keyfile();

#endif

/* vim: set shiftwidth=8 softtabstop=0 cindent cinoptions={1s: */
